% Matlab Mistuning Code: Version 1.0
%  
% Created by: Benjamin Shapiro, 
%             Control and Dynamical Systems
%             California Institute of Technology
%             Pasadena, CA 91125.
%             bshapiro@indra.caltech.edu
% 
% *********************************************************
% 
% File: model.m 
% Purpose: User defined model. Code computes derivatives
%    with respsect to mistuning. 
% Last Update: 04/22/98 by B.Shapiro
%
% *********************************************************
%

% Start screen output: 
  fprintf('Generating model defined in "model.m" ...')

% -------------------------------------------------------------
% BEGIN USER: Please pick number of blades and states per blade.
  r = 11;    % number of blades.
  d = 3;     % states per blade.
% END USER.
% -------------------------------------------------------------


  % Define symbolic variables: Mi,zi, beta
    syms beta;
    for i = 1:r
      eval(['syms z' num2str(i)]);    % Mistuning vector
      eval(['syms M' num2str(i)]);    % Component matrices
    end;

  % Define mistuning vector z
    z = z1;
    for i = 2:r
      z = [z, eval(['z' num2str(i)]) ];
    end;

% -------------------------------------------------------------
% BEGIN USER: Please input (d by d) modeling matrices Mi 
% plus first block of forcing vector beta, here.
% Mi's form the first row of circulant model matrix M.
  % Example syntax:
  %
  %          M1 = [ 0          -1;
  %                 z1    z5*z2^4] 
  %
  % Current matrix input: 24/03/98 

    % Model Parameters: Here r = 11, d = 3.
    k = 150000;           % blade stiffness
    m = 0.05;             % blade mass
    rho = 1.23;           % air density
    alpha = 0.1;          % angle of attack
    ch = 0.080;           % blade chord 
    el = 0.078;           % effective blade length
    y = 1/2;              % lag weight (if one recover unlagged)
    tau = 3.3333;         % lag time.
    wn = sqrt(k/m);       % natural freq.
    L = (rho*ch^2)/(2*m); % lift coeff.
    U = 23.1579;          % loading.
 
    % Brianno's aerodynamic coeff.
    e = [6.3573 -0.3316 0.1062 0.0958 0.0668 0.0325 ...
         -0.0054 -0.0551 -0.1413 -0.3398 -1.4943];
    f = [0.2196 1.1274 0.0452 -0.0035 -0.0050 -0.0125 ...
         -0.0128 -0.0233 -0.0428 -0.1295 -1.1629];
    
 
    % Diagonal block:
    M1 = [              0                           1               0;
          -(1+z1)/(U^2)+el*L*sin(alpha)*f(1)  -el*L*y*e(1)  -el*L*e(1)*(1-y);
                        0                      2*U*wn/tau     -2*U*wn/tau   ];
 
    % Off-diagonal blocks:
    for j = 2:r
      Mtemp = [         0                0                0; 
               f(j)*el*L*sin(alpha) -e(j)*el*L*y  -e(j)*el*L*(1-y);
                        0                0                0       ];
      eval(['M' num2str(j) ' = Mtemp;'])
    end
 
    % Forcing vector
    beta = [ 0;
             0.1*el*L*pi;
             0];
  
    % Please clear intermediate constants to avoid later conflicts:
    clear k m rho alpha ch el y tau wn U L e f;

 
% To speed computation, is your model linear in z ? 
% If Yes/No set linmodel to 1/0.
  linmodel = 1;

% END USER.
% -------------------------------------------------------------

  % Clear space for data structures:
  if hots & ~linmodel  % When using higher-orders and nonlinear model.
    DDDMrow = zeros(d,r*d,r,r,r); % d^3 Mrow/dzj dzk dzl: r by r by r block rows.
    DDDbeta = zeros(d,1,r,r,r);   % d^3 beta/dzj dzk dzl: r by r by r vectors.
  end

  if ~linmodel         % When nonlinear model 
    DDMrow = zeros(d,r*d,r,r);    % d^2 Mrow/dzj dzk: r by r const. rd by r rows.
    DDbeta = zeros(d,1,r,r);      % d^2 beta/dzj dzk: r by r const. d vectors.
  end                  

    DMrow = zeros(d,r*d,r);       % dMrow/dzj: r constant, rd by d block rows.
    Dbeta = zeros(d,1,r);         % dbeta/dzj: r constant, d vectors.

    MrowTun = zeros(d,r*d);       % Mrow(0): rd by d constant block row.
    betaTun = zeros(d,1);         % beta(0): d sized column constant vector.

    Zero = zeros(1,r);            % z = 0: the zero vector.
    % Allocate largest first to optimize memory storage.


% Define symbolic M block row, Mrow:
  Mrow = M1;
  for j = 2:r
    Mrow = [Mrow, eval(['M' num2str(j)]) ];
  end;

% Tuned matrix row MrowTun:
  MrowTun(:,:) = double( subs( Mrow, z,Zero ) );
  betaTun(:,:) = double( subs( beta, z,Zero ) );
  % double converts from symbolic to numeric.


% First order Mrow,beta derivatives:
  % Do: dMrow/dzj
  for j = 1:r                          
    DMrow(:,:,j) = double( subs( diff( Mrow, z(j) ), z, Zero ) );
    Dbeta(:,:,j) = double( subs( diff( beta, z(j) ), z, Zero ) );
  end 
  % End: dMrow/dzj                                  


% Second order Mrow derivatives
% Compute if model nonlinear
if linmodel == 0
   % Do: d Mrow/dzj
   for j = 1:r                          
     % If d/dzj const (M & beta) => 2nd derivatives zero.
     if diff( [Mrow beta], z(j)) == [DMrow(:,:,j) Dbeta(:,:,j)] 
     else
       % Do: d^2 Mrow/dzj dzk 
       for k = j:r                       
         DDMrow(:,:,j,k) =  double( subs( diff( diff(Mrow,z(j)), z(k)), z,Zero) );
         DDbeta(:,:,j,k) =  double( subs( diff( diff(beta,z(j)), z(k)), z,Zero) );
       end
       % End: d^2 Mrow/dzj dzk                               
     end 
     % End if
   end           
   % End: d Mrow/dzj
end
% End computation
% Remember: always use j =< k, eg: DDMrow(:,:,1,2).


% Third order Mrow derivatives
% Compute if model nonlinear
if hots & linmodel == 0   % If higher-order terms and nonlinear model.
   % Do: d Mrow/dzj
   for j = 1:r 
     % If d/dzj const (M & beta) => 2nd derivatives zero.
     if diff( [Mrow beta], z(j)) == [DMrow(:,:,j) Dbeta(:,:,j)]
     else
       % Do: d^2 Mrow/dzj dzk   
       for k = j:r 
         % If [dMrow/dzj](z) const => 2nd der zero.
         if diff( diff([Mrow beta],z(j)), z(k)) == [DDMrow(:,:,j,k) DDbeta(:,:,j,k)] 
         else
           % Do: d^3 Mrow/dzj dzk dzl
           for l = k:r   
             DDDMrow(:,:,j,k,l) = ...
                double( subs( diff( diff( diff(Mrow,z(j)), z(k)) ,z(l)), z,Zero) );
             DDDbeta(:,:,j,k,l) = ...
                double( subs( diff( diff( diff(beta,z(j)), z(k)) ,z(l)), z,Zero) );
           end 
           % End: d^3 Mrow/dzj dzk dzl  
         end
         % End if     
       end  
       % End: d^2 Mrow/dzj dzk  
     end
     % End if      
   end 
   % End: d Mrow/dzj
end
% End computation
% Remember: always use j =< k =< l, eg: DDDMrow(:,:,2,3,3).
% Note: not operator (~=) doesn't work with symbolics, use if/else.


% Save the following structures: 
  savedon = date;
  if hots & ~linmodel     % nonlinear and higher-order -> up to DDD.
    save('Data/modelvar','r','d','z','Zero','Mrow','beta',...
                         'MrowTun','DMrow','DDMrow','DDDMrow',...
                         'betaTun','Dbeta','DDbeta','DDDbeta',...
                         'linmodel','savedon','hots')
  elseif ~hots & ~linmodel  % just nonlinear -> up to DD.
    save('Data/modelvar','r','d','z','Zero','Mrow','beta',...
                         'MrowTun','DMrow','DDMrow',...
                         'betaTun','Dbeta','DDbeta',...
                         'linmodel','savedon','hots')
  else                   % linear -> only D.
    save('Data/modelvar','r','d','z','Zero','Mrow','beta',...
                         'MrowTun','DMrow',...
                         'betaTun','Dbeta',...
                         'linmodel','savedon','hots')    
  end


% DESCRIPTION OF DATA STRUCTURES:
%   1) Mrow: Symbolic first block row of model matrix M, contains
%            symbolic submatrices M1,M2,...,Mr defined by user.
%      Size: r d (columns) by d (rows).
%      Syntax: Mrow(i,j) is ith (row) jth (column) of Mrow.
%
%   2) MrowTun: Constant tuned first block row of model matrix,
%            so MrowTun = Mrow(z=0).
%      Size: r d (columns) by d (rows).
%      Syntax: MrowTun(i,j) is ith (row) jth (column) element
%            of MrowTun.
%
%   3) DMrow: Constant first order derivative of Mrow at z = 0,
%            specifically: DMrow = [dMrow/dz1, ... , dMrow/dzr].
%      Size: r components, each has r d (columns) by d (rows).
%      Syntax: DMrow(i,j,k) is ith (row) and jth (column) element
%            of dMrow/dzk.
%
%   4) DDMrow: Constant second order derivative of Mrow at z = 0,
%            specifically it is all combinations of: d^2 Mrow/dzk dzl.
%      Size: r by r components, each has r d (columns) by d (rows).
%      Syntax: DMrow(i,j,k,l) is ith (row) and jth (column) element
%            of d^2 Mrow/dzk dzl. Only need/valid for k =< l.
%
%   5) DDDMrow: Constant third order derivative of Mrow at z = 0,
%            all combinations of: d^3 Mrow/dzk dzl dzm.
%      Size: r by r by r components, each has r d (columns) by d (rows).
%      Syntax: DMrow(i,j,k,l,m) is ith (row) and jth (column) element
%            of d^3 Mrow/dzk dzl dzm. Only need/valid for k =< l =< m.
%
%   6) betaTun, Dbeta, DDbeta, DDDbeta: Derivatives of forcing vector
%            beta, equivalent to DMrow, DDMrow, DDDMrow structures.
%            Comment: Only need DDbeta, DDDbeta if hots = 1. If this 
%            creates big time delay remove from initialization and loops
%            for hots = 0. (Right now evaluating DDbeta for hots = 0.)
%      Size: 0, r, r*r and r*r*r components, each a d by 1 column vector.


% Finish screen output:
  fprintf(' done! \n')




