% Matlab Mistuning Code: Version 2.0
%
% Created by: Benjamin Shapiro,
%             Control and Dynamical Systems
%             California Institute of Technology
%             Pasadena, CA 91125.
%             bshapiro@indra.caltech.edu
%
%   Copyright (c) 1998, California Institute of Technology.
%   Permission is granted to copy, modify, and redistribute
%   this software provided all header messages are retained.
%
% *********************************************************
%
% File: forcing.m
% Purpose: Given spatial forcing mode l computes mistuned
%   forced response: X(z,w) = Inv[iwI-M(z)] Bl(z).
% Last Update: 05/01/98 by B.Shapiro
%
% *********************************************************

% Read in spatial forcing mode.
  fprintf('\n');
  fprintf('Please enter spatial forcing mode. It should vary between 0 and %g. \n',r-1);
  fprintf('(Note: -p nodal diameter forcing is given by l = %g-p.) \n',r);
  fprintf('\n');
  clear l;
    l = input('Spatial forcing mode: l = ');
    fprintf('\n');
    % Check input ok.
      while mod(l,1) ~= 0 | l < 0 | l > r-1
        fprintf('ERROR! Spatial forcing mode must be an integer between 0 and %g: \n',r-1);
        l = input('       l = ');
        fprintf('\n');
      end
    % End check.
  if l == 0
    l = r; % Convert to my indexing for 0ND.
  end
% Got spatial forcing mode: l between 1 and r.


% ---------------------------------------------------------------------
%   NOTE: Please see paper:                                           |
%    "Solving for Mistuned Forced Response by Symmetry", B. Shapiro   |
%          (accepted Journal of Propulsion and Power 06/98)           |
%   for a description of quantities g, gbar and Psi.                  |
% ---------------------------------------------------------------------


  % Find g0(j) = g_j(0).    % j between 1 and rd refers to jth mode.
    g0 = zeros(r*d,1);      % Only the forced mode l is active.
    g0((l-1)*d+1: l*d) = r*UTun( (l-1)*d+1:l*d , 1:d )*betaTun;
  % Got g0.


  % Find gbar(j)          % j between 1 and rd refers to jth mode.
    gbar = zeros(r*d,1);  % initialize.
    for q = 0:r-1
      topow = kron(1:r, ones(1,d) ) - l*ones(1,d*r); % [jth(1)-l, ..., jth(rd)-l].
      prow = p(q,r).^topow;                          % vector of p( (jth(j)-l)q ).
      DiagP = diag(prow);                            % Create diagonal matrix.
      deltaq = DiagP * ( Du(:,:,1+q)*betaTun + UTun(:,1:d)*Dbeta(:,1,1+q) );
      gbar = gbar + deltaq;         % contribution to gbar from qth sum component.
    end
  % Got gbar.


  % Find Psi0, DPsi: Psi(z) = Psi0 + DPsi.z + ... (for blade one).
    Psi0 = zeros(d,r*d);     % jth d-sized column for jth mode.
    DPsi = zeros(d,r,r*d);   % (:,i,j) is dPsi_j(z)/dzi.
    PSI0 = zeros(r*d,r*d);   % same as Psi0 but for ALL blades.
    DPSI = zeros(r*d,r,r*d); % same as DPsi but for ALL blades.
    % Psi0
      for j = (l-1)*d+1: l*d
        Psi0(:,j) = g0(j)*VTun(1:d,j);  % got Psi(0) for jth mode.
        PSI0(:,j) = kron( ( p(l,r).^(0:r-1) ).' , Psi0(:,j) );
           % Applied symmetry to find PSI0 = [Psi(0) for all blades].
      end
    % DPsi
      for j = 1:r*d
        for i = 1:r
          DPsi(:,i,j) = g0(j)*Dv(:,j,i)+gbar(j)*p((l-jth(j,d))*(i-1),r)*VTun(1:d,j);
          % got dPsi/dzi for jth mode.
        end 
          % Construct DPSI for jth mode
          % [Need DPsi(:,:,j) computed above.]
            for q = 0:r-1
              DPSI(q*d+1:(q+1)*d,:,j) = p(l*q,r) * rot( DPsi(:,:,j),r-q,1,r );
            end            
          % Got DPSI.
      end
  % Got Psi0, PsiLin


% Print completed calc.
  fprintf('Completed forced response calculations. \n');


  % Print (new) output to file:
    str = ['Outputs/RespData/X:l=' num2str(l) ',mode=1,1'];
    if exist(str) == 2
      fprintf('\t Warning: Data already exists for spatial forcing mode %g.\n \t',l);
      str = input(' Do you wish to overwrite this old data (y/n): ','s');
        if str == 'y'
             if unix_machine == 0
               fprintf(' Please delete all files: Outputs/RespData/X:l=%g,mode=* \n',l);
               input(' Press "enter" when done. \n');
             end
           force_out;
        end
    else
      force_out;
    end


% Save data:
  savedon = date;
  save('Outputs/forced','l','Psi0','PSI0','DPSI','savedon');
% End save.






% DESCRIPTION OF DATA STRUCTURES:
%   1) g0: Tuned contribution from left eigenvectors and
%          forcing vector beta: g0 = g(z=0).
%      Size: r d column vector.
%      Syntax: g0(j) gives contribution of jth mode/eigenvalue.
% 
%   2) gbar: Linear coefficients of g(z) (only need one by symmetry for
%          each mode/eigenvalue j). Here gbar = dg/dz1.
%      Size: r d column vector.
%      Syntax: gbar(j) gives contribution of jth mode/eigenvalue.
%
%   3) Psi0: Tuned contribution from left, right eigenvectors
%          and forcing vector beta: Psi0 = Psi(z=0) = g(z) v(z).
%          Holds only for blade one (get all blades by symmetry).
%      Size: d by rd matrix.
%      Syntax: Psi0(i,j) is ith element of contribution from jth mode.
%
%   4) PSI0: Same as Psi0 except holds for ALL blades. 
%      Size: rd by rd matrix.
%      Syntax: PSI0(i,j) is ith element of contribution from jth mode.
%          First blade: i between 1 and d; second blade: i between d+1 
%          and 2d; ... rth blade: i between (r-1)d+1 and rd.
%
%   5) DPsi: Linear coefficients of Psi(z), here DPsi(:,q,:) = dPsi/dzq.
%          Holds only for blade one (get all blades by symmetry).
%      Size: d by r by rd array.
%      Syntax: DPsi(i,q,j) is ith element of contribution from jth mode
%          of dPsi/dzq.
%
%   6) DPSI: Same as DPsi except holds for ALL blades.
%      Size: rd by r by rd array.
%      Syntax: DPSI(i,q,j) is ith element of contribution from jth mode
%          of dPsi/dzq. First blade: i between 1 and d; second blade: 
%          i between d+1 and 2d; ... rth blade: i between (r-1)d+1 and rd.






















