% Matlab Mistuning Code: Version 2.0
%
% Created by: Benjamin Shapiro,
%             Control and Dynamical Systems
%             California Institute of Technology
%             Pasadena, CA 91125.
%             bshapiro@indra.caltech.edu
%
%   Copyright (c) 1998, California Institute of Technology.
%   Permission is granted to copy, modify, and redistribute
%   this software provided all header messages are retained.
%
% *********************************************************
%
% File: perturb.m
% Purpose: Construct 2nd/1rst (hots=0) or 3rd/2nd (hots=1)
%    perturbations of eigenvalues & left/right eigenvectors.
% Last Update: 04/22/98 by B.Shapiro
%
% *********************************************************

fprintf('\n')
fprintf('Building eigenvalue/vector perturbations ... \n')

% Compose matrix derivatives:
  
  % Build dMdz1
    dMdz1 = DMrow(:,:,1);
    for j = 1:r-1
      dMdz1 = [ dMdz1;
                rot( DMrow(:,:,r-j+1) ,r-j,d,r) ];
    end
    % Diff by: Block row 2-> dzr, 3-> dz(r-1), ..., r-> dz2.
    % Rotate by: Brow 2-> r-1

  % Build ddMddz1
  if ~linmodel
    ddMddz1 = DDMrow(:,:,1,1);
    for j = 1:r-1
      ddMddz1 = [ ddMddz1;
                  rot( DDMrow(:,:,r-j+1,r-j+1) ,r-j,d,r) ];
    end
    % Diff by: Block row 2-> dzr, 3-> dz(r-1), ..., r-> dz2.
    % Rotate by: Brow 2-> r-1
  end

  % Build ddMdz1dz(1+q)
  if ~linmodel
    ddMdz1dz1pq = zeros(r*d,r*d,r-1); % Clear space.
    for q = 1:r-1                     % Do: dz1dz(1+q)
      ddMdz1dz1pq(:,:,q) =  DDMz1z1pq( DDMrow,q,d,r);
    end                               % End: dz1dz(1+q)
  end

% End Matrix derivatives



% Define K constant, as in c1, c2, ..., cK.
  K = floor( r/2 );


% Initialize variables:
  % Clear space (for things in loops)
    b = zeros(r*d,1);
    bsym = zeros(r*d,1);
    basym = zeros(r*d,1);
    c = zeros(r*d,K);
    csym = zeros(r*d,K);
    casym = zeros(r*d,K);
    Dv = zeros(d,r*d,r);
    Du = zeros(r*d,d,r);
% End initialize.


% COMPUTE aj COEFFICIENTS:
  UdMdz1V = UTun*dMdz1*VTun;
  a = diag( UdMdz1V );
  % Need the off-diagonal terms for bj's.
% DONE WITH aj COEFFICIENTS.


% COMPUTE bj, (ci)j COEFFICIENTS
% First: compute their symmetric parts (for nonlinear model)
  if ~linmodel
    bsym = diag( UTun*ddMddz1*VTun ); % symmetric bj portion
    for i = 1:K
      csym(:,i) = diag( UTun* ddMdz1dz1pq(:,:,i) *VTun  ); 
    end
  end
  % else its bsym,csym = 0 as above.

% Second: compute their asymmetric portions (even if linear model):
  % Start j loop
  for j = 1:r*d
      basym(j,1) = SumUMV( UdMdz1V, EigenTun, j, r*d );
      b(j,1) = ( bsym(j) + basym(j) )/2;

    % Start i loop
    for i = 1:K 
      casym(j,i) = SumPkUMV( UdMdz1V, EigenTun, j, i, d, r);
      c(j,i) = csym(j,i) + casym(j,i);
    end
    % End i loop
  end
  % End j loop
% DONE WITH bj, (ci)j COEFFICIENTS.
fprintf('\t Got first/second order eigenvalue coefficients. \n')



% COMPUTE Dv,Du EIGENVECTOR DERIVATIVES
  for j = 1:r*d
    for i = 1:r
      Dv(:,j,i) = SumUMVvk(VTun,UdMdz1V,EigenTun,j,i,d,r); % dvj/dzi
      Du(j,:,i) = SumUMVuk(UTun,UdMdz1V,EigenTun,j,i,d,r); % duj/dzi
    end
  end
% DONE WITH Dv, DU EIGENVECTOR DER.
fprintf('\t Got first order left/right eigenvector coefficients. \n')






   % Save results:
     savedon = date;
     save('Outputs/PertEig','a','b','c','Du','Dv','K','savedon');
   % End save.




% DESCRIPTION OF DATA STRUCTURES:
%   1) a: aj is linear perturbation coeff. of jth eigenvalue:
%         lambda_j(z) = lambda_j(0) + aj [Sum z_i] + ...
%      Size: rd column vector.
%      Syntax: a(j,1) is linear coeff. of jth eigenvalue.
%
%   2) b: bj is quadratic homogeneous perturbation coeff. of jth eigenvalue:
%         lambda_j(z) = ... + bj [Sum (z_i)^2] + ...
%           It is composed of a symmetric part (bsym) that depends on
%           d^2 M/d zi^2, and an asymmetric part (basym) which depends 
%           on d M/d zi and the coupling of adjacent eigenvalues.
%      Size: rd column vector.
%      Syntax: b(j,1) is homog. quad. coeff. of jth eigenvalue.
%
%   3) c: (cl)j is the lth quadratic mixed perturbation coeff. of 
%         the jth eigenvalue:
%         lambda_j(z) = ... + cl [Sum z_i z_(i+l)] + ...
%           It is composed of a symmetric part (csym) that depends on
%           d^2 M/dzi dz(i+l), and an asymmetric part (casym) which  
%           depends on d M/d zi and the coupling of adjacent eigenvalues.
%         Here l can vary between 1 and K = floor(r/2).
%      Size: rd by K matrix.
%      Syntax: c(j,l) is lth mixed quad. coeff. of jth eigenvalue.
%
%   4) Dv: dvj/dzi is the derivative of the first block of the jth
%          right eigenvector with respect to zi. Dv collects all 
%          the possible permutations with respect to j and i.
%      Size: d by rd by r matrix.
%      Syntax: dvj/dzi = Dv(:,j,i), d sized column vector.
%
%   5) Du: duj/dzi is the derivative of the first block of the jth
%          left eigenvector with respect to zi. Du collects all 
%          the possible permutations with respect to j and i.
%      Size: rd by d by r matrix.
%      Syntax: dvj/dzi = Du(j,:,i), d sized row vector.
%



